/*
 * Copyright (C) Sergey P. Derevyago, 2003-2004.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 *   FindFile.
 */

#ifndef __FINDFILE_HPP__
 #define __FINDFILE_HPP__

#include "cexception.hpp"

/**
 *    callback   FindFile.
 */
class FindFileCallback {
 public:
      /**
       *        FindFile::find() 
       *   .    false,  FindFile::find()
       *   . ,  found()  
       *    FindFile::find(),    
       *   .
       */
      virtual bool found(const std::string& fileName)=0;

      /**
       *   .
       */
      virtual ~FindFileCallback() {}

 private:
      /**   */
      FindFileCallback& operator=(const FindFileCallback&);
};

/**
 *          
 *    .
 */
class FindFile {
 public:
      /**
       *      mask   dir ,  
       *  recur,    .    
       *   found()   FindFileCallback. 
       *   false,       find() 
       *  false.  CException   .
       */
      virtual bool find(const std::string& dir, bool recur,
        const std::string& mask, FindFileCallback& cb)=0;

      /**
       *   .
       */
      virtual ~FindFile() {}

 private:
      /**   */
      FindFile& operator=(const FindFile&);
};

/**
 *  .
 */
namespace Factory {

 /**
  *  ,   FindFile.    
  *   CException.
  */
 sh_ptr<FindFile> newFindFile();

}

#endif
