/*
 * Copyright (C) Sergey P. Derevyago, 2003-2004.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 *   FindText     .
 */

#ifndef __FINDTEXT_HPP__
 #define __FINDTEXT_HPP__

#include "pubsub.hpp"
#include "cexception.hpp"

//     FindText,   
//     
#undef FindText

class FindText;

/**
 *         
 *      .    
 *    (..    search()) 
 *   stopSearch().
 */
struct MsgBase {
       /**      */
       FindText& ft;

       /**
        *     .
        */
       MsgBase(FindText& ft_) : ft(ft_) {}
};

/**
 *     ,   
 * FindText        .
 */
struct FoundMsg : MsgBase {
       /**   */
       const std::string fileName;
       /**   ( 1) */
       const int lineNum;
       /**   */
       const std::string lineText;

       /**
        *     .
        */
       FoundMsg(FindText& ft, const std::string& file, int num,
         const std::string& txt) : MsgBase(ft), fileName(file), lineNum(num),
         lineText(txt) {}
};

/**
 *         ,
 *   FindText  .
 */
struct InfoMsg : MsgBase {
       /**   :  ,   */
       enum Type { opened, closed };

       /**   */
       const Type type;
       /**   */
       const std::string fileName;

       /**
        *     .
        */
       InfoMsg(FindText& ft, Type t, const std::string& fn) : MsgBase(ft),
         type(t), fileName(fn) {}
};

/**
 *         (,
 *      )   ,
 *    FindText  .
 */
struct ErrorMsg : MsgBase {
       /**    */
       const std::string errMsg;

       /**
        *     .
        */
       ErrorMsg(FindText& ft, const std::string& err) : MsgBase(ft), errMsg(err)
         {}
};

/**
 *          
 * .        ,
 *          
 * .
 */
class FindText {
 public:
      /**
       *    text  ,   mask.
       *     dir ,    recur,  
       *  .      , 
       *       .
       *  CException     .
       */
      virtual void search(const std::string& dir, bool recur,
        const std::string& mask, const std::string& text)=0;

      /**
       *        search(). 
       *        .  
       *    CException.
       */
      virtual void stopSearch()=0;

      /**
       * ,     .  
       *       stopSearch() 
       *  .
       */
      virtual Publisher<FoundMsg>& foundPub()=0;

      /**
       * ,      . 
       *       
       * stopSearch()   .
       */
      virtual Publisher<InfoMsg>& infoPub()=0;

      /**
       * ,     ,  
       *  .       
       *  stopSearch()   .
       */
      virtual Publisher<ErrorMsg>& errorPub()=0;

      /**
       *   .
       */
      virtual ~FindText() {}

 private:
      /**   */
      FindText& operator=(const FindText&);
};

/**
 *  .
 */
namespace Factory {

 /**
  *  ,   FindText.    
  *   CException.
  */
 sh_ptr<FindText> newFindText();

}

#endif
