/*
 * Copyright (C) Sergey P. Derevyago, 2003-2004.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 *   NameMatcher.
 */

#include "namematcher.hpp"
#include <ctype.h>

using namespace std;

namespace {

 /**
  * -  NameMatcher,  
  *     .
  */
 class NameMatcherImpl : public NameMatcher {
       /** ,       */
       string suff;
       /**     */
       int send;

       /**
        *     .   
        *     "*ext" (..   
        *   "ext")    .  "*" 
        *    , ..  "ext"  "*ext".
        */
       virtual bool match(const std::string& fileName);

  public:
       /**
        *     .
        */
       NameMatcherImpl(const std::string& mask);
 };

 NameMatcherImpl::NameMatcherImpl(const std::string& mask)
 {
  int beg= (mask[0]=='*') ? 1 : 0;
  suff.reserve(mask.size()-beg);

  for (int i=beg; i<mask.size(); i++) suff.push_back(tolower(mask[i]));
  send=suff.size()-1;
 }

 bool NameMatcherImpl::match(const std::string& fileName)
 {
  if (fileName.size()<suff.size()) return 0;

  for (int fend=fileName.size()-1, i=0; i<suff.size(); i++)
      if (tolower(fileName[fend-i])!=suff[send-i]) return 0;

  return 1;
 }

}

sh_ptr<NameMatcher> Factory::newNameMatcher(const std::string& mask)
{
 try { return sh_ptr<NameMatcher>(new NameMatcherImpl(mask)); }
 catch (...) {
       throw newCException(_FLINE_, "Can't create NameMatcherImpl object",
         toCException(_FLINE_));
 }
}
