/*
 * Copyright (C) Sergey P. Derevyago, 2003-2004.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 *  -,  ,    
 *  Publisher  Subscriber.   ftext   .
 */

#include <stdio.h>
#include <string>
#include "pubsub.hpp"

using namespace std;

/**     /  */
struct Named {
       string name;
       
       Named(const string& n) : name(n)
       {
        printf("constructed: %s\n", name.c_str());
       }
       
       ~Named() { printf("destructed: %s\n", name.c_str()); }
};

/**  */
struct Pub : Named, Publisher<string> {
       Pub(const string& name) : Named(name) {}
};

/**  */
struct Sub : Named, Subscriber<string> {
       Sub(const string& name) : Named(name) {}

       virtual bool regularMsg(const Publisher<string>& pub, const string& msg)
       {
        printf("%s received %s from %s\n", name.c_str(), msg.c_str(),
          static_cast<const Pub&>(pub).name.c_str());
        return 1;
       }

       virtual void subscribedMsg(const Publisher<string>& pub)
       {
        printf("%s received subscribedMsg from %s\n", name.c_str(), 
          static_cast<const Pub&>(pub).name.c_str());
       }

       virtual void unsubscribedMsg(const Publisher<string>& pub)
       {
        printf("%s received unsubscribedMsg from %s\n", name.c_str(), 
          static_cast<const Pub&>(pub).name.c_str());
       }
};

int main()
{
 Pub p1("p1");
 {
   Sub s1("s1");
   {
     Pub p2("p2");
     {
       Sub s2("s2");

       p1.subscribe(s1);
       p1.subscribe(s2);
       p2.subscribe(s1);
       p2.subscribe(s2);

       p1.send("message1");
       p2.send("message1");
     }

     p1.send("message2");
     p2.send("message2");
    }

   p1.send("message3");
 }

 p1.send("message4");
}
